<?php

namespace Litematrimony\Profile\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litematrimony\Profile\Forms\Request as RequestForm;
use Litematrimony\Profile\Http\Requests\RequestRequest;
use Litematrimony\Profile\Interfaces\RequestRepositoryInterface;
use Litematrimony\Profile\Repositories\Eloquent\Filters\RequestResourceFilter;
use Litematrimony\Profile\Repositories\Eloquent\Presenters\RequestListPresenter;

/**
 * Resource controller class for request.
 */
class RequestResourceController extends BaseController
{

    /**
     * Initialize request resource controller.
     *
     *
     * @return null
     */
    public function __construct(RequestRepositoryInterface $request)
    {
        parent::__construct();
        $this->form = RequestForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litematrimony.profile.modules'), 'profile', guard_url('profile'));
        $this->repository = $request;
    }

    /**
     * Display a list of request.
     *
     * @return Response
     */
    public function index(RequestRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(RequestResourceFilter::class)
            ->setPresenter(RequestListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('profile::request.names'))
            ->view('profile::request.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display request.
     *
     * @param Request $request
     * @param Model   $request
     *
     * @return Response
     */
    public function show(RequestRequest $request, RequestRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('profile::request.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('profile::request.show')
            ->output();
    }

    /**
     * Show the form for creating a new request.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(RequestRequest $request, RequestRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('profile::request.name'))
            ->view('profile::request.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new request.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RequestRequest $request, RequestRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('profile::request.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('profile/request/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/profile/request'))
                ->redirect();
        }

    }

    /**
     * Show request for editing.
     *
     * @param Request $request
     * @param Model   $request
     *
     * @return Response
     */
    public function edit(RequestRequest $request, RequestRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('profile::request.name'))
            ->view('profile::request.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the request.
     *
     * @param Request $request
     * @param Model   $request
     *
     * @return Response
     */
    public function update(RequestRequest $request, RequestRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('profile::request.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('profile/request/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('profile/request/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the request.
     *
     * @param Model   $request
     *
     * @return Response
     */
    public function destroy(RequestRequest $request, RequestRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('profile::request.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('profile/request/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('profile/request/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
