<?php

namespace Litematrimony\Profile\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litematrimony\Profile\Forms\Shortlist as ShortlistForm;
use Litematrimony\Profile\Http\Requests\ShortlistRequest;
use Litematrimony\Profile\Interfaces\ShortlistRepositoryInterface;
use Litematrimony\Profile\Repositories\Eloquent\Filters\ShortlistResourceFilter;
use Litematrimony\Profile\Repositories\Eloquent\Presenters\ShortlistListPresenter;

/**
 * Resource controller class for shortlist.
 */
class ShortlistResourceController extends BaseController
{

    /**
     * Initialize shortlist resource controller.
     *
     *
     * @return null
     */
    public function __construct(ShortlistRepositoryInterface $shortlist)
    {
        parent::__construct();
        $this->form = ShortlistForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litematrimony.profile.modules'), 'profile', guard_url('profile'));
        $this->repository = $shortlist;
    }

    /**
     * Display a list of shortlist.
     *
     * @return Response
     */
    public function index(ShortlistRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(ShortlistResourceFilter::class)
            ->setPresenter(ShortlistListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('profile::shortlist.names'))
            ->view('profile::shortlist.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display shortlist.
     *
     * @param Request $request
     * @param Model   $shortlist
     *
     * @return Response
     */
    public function show(ShortlistRequest $request, ShortlistRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('profile::shortlist.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('profile::shortlist.show')
            ->output();
    }

    /**
     * Show the form for creating a new shortlist.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(ShortlistRequest $request, ShortlistRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('profile::shortlist.name'))
            ->view('profile::shortlist.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new shortlist.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ShortlistRequest $request, ShortlistRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('profile::shortlist.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('profile/shortlist/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/profile/shortlist'))
                ->redirect();
        }

    }

    /**
     * Show shortlist for editing.
     *
     * @param Request $request
     * @param Model   $shortlist
     *
     * @return Response
     */
    public function edit(ShortlistRequest $request, ShortlistRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('profile::shortlist.name'))
            ->view('profile::shortlist.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the shortlist.
     *
     * @param Request $request
     * @param Model   $shortlist
     *
     * @return Response
     */
    public function update(ShortlistRequest $request, ShortlistRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('profile::shortlist.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('profile/shortlist/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('profile/shortlist/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the shortlist.
     *
     * @param Model   $shortlist
     *
     * @return Response
     */
    public function destroy(ShortlistRequest $request, ShortlistRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('profile::shortlist.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('profile/shortlist/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('profile/shortlist/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
